@@ -48,6 +48,7 @@ const _FULLSCREEN_TOOLBAR_TIMEOUT = 2; // seconds
 
 const PreviewView = new Lang.Class({
     Name: 'PreviewView',
+    Extends: Gtk.Stack,
 
     _init: function(overlay) {
         this._model = null;
@@ -66,11 +67,11 @@ const PreviewView = new Lang.Class({
         Application.modeController.connect('window-mode-changed', Lang.bind(this,
             this._onWindowModeChanged));
 
-        this.widget = new Gtk.Stack({ homogeneous: true,
-                                      transition_type: Gtk.StackTransitionType.CROSSFADE });
+        this.parent({ homogeneous: true,
+                      transition_type: Gtk.StackTransitionType.CROSSFADE });
 
         this._errorBox = new ErrorBox.ErrorBox();
-        this.widget.add_named(this._errorBox.widget, 'error');
+        this.add_named(this._errorBox, 'error');
 
         this._sw = new Gtk.ScrolledWindow({ hexpand: true,
                                             vexpand: true });
@@ -79,7 +80,7 @@ const PreviewView = new Lang.Class({
         this._sw.get_vscrollbar().connect('button-press-event', Lang.bind(this, this._onScrollbarClick));
         this._sw.get_hadjustment().connect('value-changed', Lang.bind(this, this._onAdjustmentChanged));
         this._sw.get_vadjustment().connect('value-changed', Lang.bind(this, this._onAdjustmentChanged));
-        this.widget.add_named(this._sw, 'view');
+        this.add_named(this._sw, 'view');
 
         this._createView();
 
@@ -88,7 +89,7 @@ const PreviewView = new Lang.Class({
         this._previewContextMenu = Gtk.Menu.new_from_model(model);
         this._previewContextMenu.attach_to_widget(this._sw, null);
 
-        this.widget.show_all();
+        this.show_all();
 
         this._bookmarkPage = Application.application.lookup_action('bookmark-page');
         this._bookmarkPage.enabled = false;
@@ -136,8 +137,8 @@ const PreviewView = new Lang.Class({
             function() {
                 this._changeRotation(90);
             }));
-        let showPlaces = Application.application.lookup_action('places');
-        let showPlacesId = showPlaces.connect('activate', Lang.bind(this, this._showPlaces));
+        this._places = Application.application.lookup_action('places');
+        let placesId = this._places.connect('activate', Lang.bind(this, this._showPlaces));
 
         let nightModeId = Application.application.connect('action-state-changed::night-mode',
             Lang.bind(this, this._updateNightMode));
@@ -151,7 +152,7 @@ const PreviewView = new Lang.Class({
         Application.documentManager.connect('load-error',
                                             Lang.bind(this, this._onLoadError));
 
-        this.widget.connect('destroy', Lang.bind(this,
+        this.connect('destroy', Lang.bind(this,
             function() {
                 this._bookmarkPage.disconnect(bookmarkPageId);
                 this._zoomIn.disconnect(zoomInId);
@@ -161,7 +162,7 @@ const PreviewView = new Lang.Class({
                 this._copy.disconnect(copyId);
                 rotLeft.disconnect(rotLeftId);
                 rotRight.disconnect(rotRightId);
-                showPlaces.disconnect(showPlacesId);
+                this._places.disconnect(placesId);
                 Application.application.disconnect(presentCurrentId);
                 Application.application.disconnect(nightModeId);
             }));
@@ -169,7 +170,7 @@ const PreviewView = new Lang.Class({
 
     _onLoadStarted: function() {
         this._bookmarkPage.enabled = false;
-        this._showPlaces.enabled = false;
+        this._places.enabled = false;
         this._copy.enabled = false;
     },
 
@@ -217,12 +218,12 @@ const PreviewView = new Lang.Class({
 
     _setError: function(primary, secondary) {
         this._errorBox.update(primary, secondary);
-        this.widget.set_visible_child_name('error');
+        this.set_visible_child_name('error');
     },
 
     _showPlaces: function() {
         let dialog = new Places.PlacesDialog(this._model, this._bookmarks);
-        dialog.widget.connect('response', Lang.bind(this,
+        dialog.connect('response', Lang.bind(this,
             function(widget, response) {
                 widget.destroy();
             }));
@@ -239,7 +240,7 @@ const PreviewView = new Lang.Class({
 
     _showPresentation: function(output) {
         this._presentation = new Presentation.PresentationWindow(this._model);
-        this._presentation.window.connect('destroy', Lang.bind(this, this._hidePresentation));
+        this._presentation.connect('destroy', Lang.bind(this, this._hidePresentation));
         if (output)
             this._presentation.setOutput(output);
     },
@@ -250,7 +251,7 @@ const PreviewView = new Lang.Class({
             this._showPresentation();
         } else {
             let chooser = new Presentation.PresentationOutputChooser(outputs);
-            chooser.connect('output-activated', Lang.bind(this,
+            chooser.connectJS('output-activated', Lang.bind(this,
                 function(chooser, output) {
                     if (output) {
                         this._showPresentation(output);
@@ -361,22 +362,22 @@ const PreviewView = new Lang.Class({
             this._handleExternalLink));
 
         this._navControls = new PreviewNavControls(this, this._overlay);
-        this.widget.set_visible_child_full('view', Gtk.StackTransitionType.NONE);
+        this.set_visible_child_full('view', Gtk.StackTransitionType.NONE);
     },
 
     _getPreviewContextMenu: function() {
         let builder = new Gtk.Builder();
-        builder.add_from_resource('/org/gnome/documents/preview-context-menu.ui');
+        builder.add_from_resource('/org/gnome/Documents/ui/preview-context-menu.ui');
         return builder.get_object('preview-context-menu');
     },
 
     _syncControlsVisible: function() {
         if (this._controlsVisible) {
             if (this._fsToolbar)
-                this._fsToolbar.show();
+                this._fsToolbar.reveal();
         } else {
             if (this._fsToolbar)
-                this._fsToolbar.hide();
+                this._fsToolbar.conceal();
         }
     },
 
@@ -396,14 +397,14 @@ const PreviewView = new Lang.Class({
             // create fullscreen toolbar (hidden by default)
             this._fsToolbar = new PreviewFullscreenToolbar(this);
             this._fsToolbar.setModel(this._model);
-            this._overlay.add_overlay(this._fsToolbar.revealer);
+            this._overlay.add_overlay(this._fsToolbar);
 
-            this._fsToolbar.connect('show-controls', Lang.bind(this,
+            this._fsToolbar.connectJS('show-controls', Lang.bind(this,
                 function() {
                     this.controlsVisible = true;
                 }));
         } else {
-            this._fsToolbar.revealer.destroy();
+            this._fsToolbar.destroy();
             this._fsToolbar = null;
         }
 
@@ -528,7 +529,7 @@ const PreviewView = new Lang.Class({
         // FIXME: ev_job_find_get_results() returns a GList **
         // and thus is not introspectable
         GdPrivate.ev_view_find_changed(this.view, job, page);
-        this.emit('search-changed', job.has_results());
+        this.emitJS('search-changed', job.has_results());
     },
 
     reset: function() {
@@ -560,11 +561,13 @@ const PreviewView = new Lang.Class({
 
             let hasMultiplePages = (this._model.document.get_n_pages() > 1);
             this._bookmarkPage.enabled = hasMultiplePages && this._bookmarks;
-            this._showPlaces.enabled = hasMultiplePages;
+            this._places.enabled = hasMultiplePages;
 
             this._model.connect('page-changed', Lang.bind(this, this._onPageChanged));
 
             this._updateNightMode();
+
+            this.set_visible_child_full('view', Gtk.StackTransitionType.NONE);
         }
     },
 
@@ -587,7 +590,7 @@ const PreviewView = new Lang.Class({
         return this._lastSearch;
     }
 });
-Signals.addSignalMethods(PreviewView.prototype);
+Utils.addJSSignalMethods(PreviewView.prototype);
 
 const _PREVIEW_NAVBAR_MARGIN = 30;
 const _AUTO_HIDE_TIMEOUT = 2;
@@ -857,7 +860,7 @@ const PreviewToolbar = new Lang.Class({
         this._setToolbarTitle();
         this.toolbar.show_all();
 
-        this.widget.connect('destroy', Lang.bind(this,
+        this.connect('destroy', Lang.bind(this,
             function() {
                 this._searchAction.enabled = true;
             }));
@@ -887,7 +890,7 @@ const PreviewToolbar = new Lang.Class({
 
     _getPreviewMenu: function() {
         let builder = new Gtk.Builder();
-        builder.add_from_resource('/org/gnome/documents/preview-menu.ui');
+        builder.add_from_resource('/org/gnome/Documents/ui/preview-menu.ui');
         let menu = builder.get_object('preview-menu');
 
         let doc = Application.documentManager.getActiveItem();
@@ -931,7 +934,7 @@ const PreviewSearchbar = new Lang.Class({
 
     _init: function(previewView) {
         this._previewView = previewView;
-        this._previewView.connect('search-changed', Lang.bind(this, this._onSearchChanged));
+        this._previewView.connectJS('search-changed', Lang.bind(this, this._onSearchChanged));
 
         this.parent();
     },
@@ -982,7 +985,7 @@ const PreviewSearchbar = new Lang.Class({
         this._previewView.startSearch(this._searchEntry.get_text());
     },
 
-    show: function() {
+    reveal: function() {
         this.parent();
 
         if (!this._searchEntry.get_text()) {
@@ -994,7 +997,7 @@ const PreviewSearchbar = new Lang.Class({
         this._previewView.startSearch(this._searchEntry.get_text());
     },
 
-    hide: function() {
+    conceal: function() {
         this._previewView.view.find_set_highlight_search(false);
 
         this.searchChangeBlocked = true;
@@ -1005,14 +1008,15 @@ const PreviewSearchbar = new Lang.Class({
 
 const PreviewFullscreenToolbar = new Lang.Class({
     Name: 'PreviewFullscreenToolbar',
-    Extends: PreviewToolbar,
+    Extends: Gtk.Revealer,
 
     _init: function(previewView) {
-        this.parent(previewView);
+        this.parent({ valign: Gtk.Align.START });
 
-        this.revealer = new Gtk.Revealer({ valign: Gtk.Align.START });
-        this.revealer.add(this.widget);
-        this.revealer.show();
+        this._toolbar = new PreviewToolbar(previewView);
+
+        this.add(this._toolbar);
+        this.show();
 
         // make controls show when a toolbar action is activated in fullscreen
         let actionNames = ['gear-menu', 'search'];
@@ -1025,13 +1029,13 @@ const PreviewFullscreenToolbar = new Lang.Class({
                     function(actionGroup, actionName, value) {
                         let state = value.get_boolean();
                         if (state)
-                            this.emit('show-controls');
+                            this.emitJS('show-controls');
                     }));
 
                 signalIds.push(signalId);
             }));
 
-        this.widget.connect('destroy', Lang.bind(this,
+        this._toolbar.connect('destroy', Lang.bind(this,
             function() {
                 signalIds.forEach(
                     function(signalId) {
@@ -1040,13 +1044,21 @@ const PreviewFullscreenToolbar = new Lang.Class({
             }));
     },
 
-    show: function() {
-        this.revealer.set_reveal_child(true);
+    handleEvent: function(event) {
+        this._toolbar.handleEvent(event);
     },
 
-    hide: function() {
-        this.revealer.set_reveal_child(false);
+    setModel: function(model) {
+        this._toolbar.setModel(model);
+    },
+
+    reveal: function() {
+        this.set_reveal_child(true);
+    },
+
+    conceal: function() {
+        this.set_reveal_child(false);
         Application.application.change_action_state('search', GLib.Variant.new('b', false));
     }
 });
-Signals.addSignalMethods(PreviewFullscreenToolbar.prototype);
+Utils.addJSSignalMethods(PreviewFullscreenToolbar.prototype);
